<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use FiloBlu\Refilo\Model\Config\Seo\HrefLangConfig;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\Service\StoreConfigManager;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Store\Model\StoreScopeProvider;
use MageWorx\SeoBase\Helper\Hreflangs;
use Psr\Log\LoggerInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class Urls
 * @package FiloBlu\Refilo\Model\Exporter
 */
class Urls
{
    /**
     * @var Hreflangs
     */
    protected $hreflangHelper;
    /**
     * @var StoreConfigManager
     */
    protected $storeConfigManager;
    /**
     * @var array|null
     */
    protected $storeUrls;
    /**
     * @var array|null
     */
    protected $storeBaseUrls;
    /**
     * @var array|null
     */
    protected $storeLocales;
    /**
     * @var array|null
     */
    protected $seoBaseLocales;
    /**
     * @var string | null
     */
    protected $homeIdentifier;
    /**
     * @var HrefLangConfig
     */
    protected $hrefLangConfig;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var Mongo
     */
    private $mongo;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var array
     */
    private $hrefLangConfigLookup;

    /**
     * Urls constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param StoreManagerInterface $storeManager
     * @param StoreConfigManager $storeConfigManager
     * @param Hreflangs $hreflangs
     * @param LoggerInterface $logger
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        StoreManagerInterface $storeManager,
        StoreConfigManager $storeConfigManager,
        Hreflangs $hreflangs,
        LoggerInterface $logger,
        HrefLangConfig $hrefLangConfig
    )
    {
        $this->resourceConnection = $resourceConnection;
        $this->scopeConfig = $scopeConfig;
        $this->mongo = $mongo;
        $this->storeManager = $storeManager;
        $this->storeConfigManager = $storeConfigManager;
        $this->hreflangHelper = $hreflangs;
        $this->logger = $logger;
        $this->hrefLangConfig = $hrefLangConfig;
    }

    /**
     * @return array
     */
    public function getUrls()
    {
        $tableName = $this->resourceConnection->getTableName('url_rewrite');
        $query = "SELECT {$tableName}.*,store.code as store_code FROM {$tableName} INNER JOIN store on store.store_id = {$tableName}.store_id WHERE {$tableName}.store_id != 0";
        return $this->resourceConnection->getConnection()->fetchAll($query, [], \Zend_Db::FETCH_ASSOC) ?? [];
    }

    /**
     * @throws \MongoDB\Driver\Exception\Exception
     */
    public function exportUrls(OutputInterface $output = null)
    {
        $this->init();
        $result = $this->getUrls();
        $lookupRewrites = [];
        $bulkData = [];

        foreach ($result as $item)
        {
            $targetPath = $item['target_path'];

            if(!isset($lookupRewrites[$targetPath])) {
                $lookupRewrites[$targetPath] = [];
            }

            $lookupRewrites[$targetPath][] = $item;
        }

        foreach ($result as $item) {
            $bulkData[] = [
                '_id'         => $item['url_rewrite_id'],
                'url_id'      => $item['url_rewrite_id'],
                'type'        => $item['entity_type'] === 'contenttype_content' ? 'master_data' : $item['entity_type'],
                'entity_id'   => $item['entity_id'],
                'url'         => $item['request_path'],
                'is_redirect' => $item['redirect_type'],
                'target_url'  => $item['target_path'],
                'store'       => $item['store_code'],
                'hreflang'    => $this->getHreflang($item['target_path'], $item['request_path'], $item['entity_type'], $lookupRewrites)
            ];
        }

        $database = $this->scopeConfig->getValue('mongo/general/mongo_general_database');
        if (!empty($bulkData)) {

            $this->mongo->dropCollection('urls', $database);

            try {
                $this->mongo->update('_id', $bulkData, 'urls', true, $database);
                $this->mongo->ensureIndex(['url'], 'urls');
            } catch (\Throwable $throwable) {
                $this->logger->error($throwable->getMessage(), ['exception' => $throwable]);
                if($output) {
                    $output->writeln($e->getMessage());
                }
            }

        } else {
            try {
                $this->mongo->flushCollection('urls', $database);
            } catch (\Throwable $throwable) {
                $this->logger->error($throwable->getMessage(), ['exception' => $throwable]);
                if($output) {
                    $output->writeln($e->getMessage());
                }
            }
        }
    }

    /**
     * Init Function
     */
    protected function init()
    {
        $this->seoBaseLocales = [];
        $this->storeUrls = [];
        $this->storeBaseUrls = [];
        $this->hrefLangConfigLookup = [];
        $locale = '';
        $addCountryCode = false;

        foreach ($this->storeManager->getStores() as $store) {
            $this->storeUrls[(int)$store->getId()] = $store->getBaseUrl();
            $this->storeBaseUrls[(int)$store->getId()] = $store->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_DIRECT_LINK);
            $this->seoBaseLocales[$store->getCode()] = $this->hrefLangConfig->getSeoLocaleCodeForStore($store);
            $this->hrefLangConfigLookup[(int)$store->getId()] = $this->hrefLangConfig->getConfig($store);
        }

        $this->storeLocales = [];
        foreach ($this->storeConfigManager->getStoreConfigs() as $storeConfig) {
            $this->storeLocales[$storeConfig->getCode()] = $storeConfig->getLocale();
        }

        $this->homeIdentifier = $this->scopeConfig->getValue('web/default/cms_home_page');
    }

    /**
     * @param $targetPath
     * @param $requestPath
     * @param $type
     * @param $result
     * @return array
     */
    private function getHreflang($targetPath, $requestPath, $type, $result)
    {
        $hrefArray = [];
        $isHomePage = $requestPath === $this->homeIdentifier;

        foreach ($result[$targetPath] as $url) {
            //If Disabled General HrefLang
            if(!$this->hrefLangConfigLookup[(int)$url['store_id']]['enabled']){
                return [];
            }
            //If Disabled Cms HrefLang
            if(!$this->hrefLangConfigLookup[(int)$url['store_id']]['enabled_cms'] && $type === "cms-page"){
                return [];
            }
            //If Disabled Category HrefLang
            if(!$this->hrefLangConfigLookup[(int)$url['store_id']]['enabled_category'] && $type === "category"){
                return [];
            }
            //If Disabled Product HrefLang
            if(!$this->hrefLangConfigLookup[(int)$url['store_id']]['enabled_product'] && $type === "product"){
                return [];
            }

            if (!isset($url['store_code'], $url['store_id']) || (int)$url['store_id'] === 0) {
                continue;
            }

            if ($isHomePage && $type === 'cms-page') {
                $hrefArray['x-default'] = [
                    'code' => 'x-default',
                    'url' => $this->storeBaseUrls[$url['store_id']]
                ];
                $hrefArray[$url['store_code']] = [
                    'code' => ($this->hrefLangConfigLookup[(int)$url['store_id']]['use_magento_lang_code'])?$this->storeLocales[$url['store_code']]:$this->seoBaseLocales[$url['store_code']],
                    'url' => $this->storeUrls[$url['store_id']]
                ];
            }

            if (!$isHomePage && $url['target_path'] === $targetPath) {
                $hrefArray[$url['store_code']] = [
                    'code' => ($this->hrefLangConfigLookup[(int)$url['store_id']]['use_magento_lang_code'])?$this->storeLocales[$url['store_code']]:$this->seoBaseLocales[$url['store_code']],
                    'url' => "{$this->storeUrls[$url['store_id']]}{$url['request_path']}"
                ];
            }
        }

        return $hrefArray;
    }
}
